#!/bin/bash

MDE_CHAIN=mdechain
MDE_PACKET_STAMP="-m mark ! --mark 0x1/0x1"
MDE_NFQUEUE_BYPASS="NFQUEUE --queue-num 0 --queue-bypass"
MDE_TCP_PACKET_FILTER="-p tcp --tcp-flags FIN,SYN,RST,ACK,PSH SYN"
MDE_UDP_PACKET_FILTER="-p udp"
MDE_IPTABLES_SAVE_FILE="/opt/microsoft/mdatp/conf/mde_isolate_device_iptable_rules.txt"

if [ "$(id -u)" != "0" ] ; then
   logger -p daemon.err -t "setup_iptables_rules" "User has insufficient privilege"
   echo "setup_iptables_rules" "User has insufficient privilege"
   exit 4
fi

run_or_fail() {
   cmd="$@"
   status="0"
   out=$($cmd 2>&1) || status="$?"
   
   if [ "$status" != "0" ]; then
      cmd=$(echo $cmd | /bin/sed 's/[[:space:]]/ /g')
      logger -p daemon.err -t "setup_iptables_rules" "Failed to run: \"$cmd\", error code: $status, \"$out\""
      echo "setup_iptables_rules" "Failed to run: \"$cmd\", error code: $status, \"$out\""
      exit 4
   fi
}

run_or_warn() {
   cmd="$@"
   status="0"
   out=$($cmd 2>&1) || status="$?"
   
   if [ "$status" != "0" ]; then
      cmd=$(echo $cmd | /bin/sed 's/[[:space:]]/ /g')
      logger -p daemon.warn -t "cleanup_iptables_rules" "\"$cmd\" returns with code: $status, message: \"$out\""
      echo "cleanup_iptables_rules" "Failed to run: \"$cmd\", error code: $status, \"$out\""
      return 4
   fi
}

setup_iptable_rules() {
   echo "Setting up IPtables rules"
   run_or_fail "iptables -N ${MDE_CHAIN}"
    
   #Input chain rule
   run_or_fail "iptables -I INPUT ${MDE_PACKET_STAMP} ${MDE_TCP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_fail "iptables -I INPUT ${MDE_PACKET_STAMP} ${MDE_UDP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_fail "iptables -I INPUT --jump ACCEPT --protocol tcp --dport 53"
   run_or_fail "iptables -I INPUT --jump ACCEPT --protocol udp --dport 53"

   #Output chain rule
   run_or_fail "iptables -I OUTPUT ${MDE_PACKET_STAMP} ${MDE_TCP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_fail "iptables -I OUTPUT ${MDE_PACKET_STAMP} ${MDE_UDP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_fail "iptables -I OUTPUT -m owner --gid-owner mdatp -j ACCEPT"
   run_or_fail "iptables -I OUTPUT --jump ACCEPT --protocol tcp --sport 53"
   run_or_fail "iptables -I OUTPUT --jump ACCEPT --protocol udp --sport 53"
   run_or_fail "iptables -A OUTPUT --protocol icmp -j REJECT --reject-with icmp-port-unreachable"

   run_or_fail "iptables -I ${MDE_CHAIN} -j ${MDE_NFQUEUE_BYPASS}"

   # Save the rules for reboot cycle
   run_or_fail "iptables-save -f ${MDE_IPTABLES_SAVE_FILE}"
   echo "IPtables rules setup completed"
}

cleanup_iptable_rules() {
   echo "Flush IPtable rules"
   run_or_warn "iptables -D INPUT ${MDE_PACKET_STAMP} ${MDE_TCP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_warn "iptables -D INPUT ${MDE_PACKET_STAMP} ${MDE_UDP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_warn "iptables -D INPUT --jump ACCEPT --protocol tcp --dport 53"
   run_or_warn "iptables -D INPUT --jump ACCEPT --protocol udp --dport 53"

   run_or_warn "iptables -D OUTPUT ${MDE_PACKET_STAMP} ${MDE_TCP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_warn "iptables -D OUTPUT ${MDE_PACKET_STAMP} ${MDE_UDP_PACKET_FILTER} -j ${MDE_CHAIN}"
   run_or_warn "iptables -D OUTPUT -m owner --gid-owner mdatp -j ACCEPT"
   run_or_warn "iptables -D OUTPUT --protocol icmp -j REJECT --reject-with icmp-port-unreachable"
   run_or_warn "iptables -D OUTPUT --jump ACCEPT --protocol tcp --sport 53"
   run_or_warn "iptables -D OUTPUT --jump ACCEPT --protocol udp --sport 53"
   
   run_or_warn "iptables -D ${MDE_CHAIN} -j ${MDE_NFQUEUE_BYPASS}"
   run_or_warn "iptables -X ${MDE_CHAIN}"
   echo "Cleaned up all IPtables rules"
}

if [ "$1" = "-i" ]; then
   setup_iptable_rules
elif [ "$1" = "-u" ]; then
   cleanup_iptable_rules
else
   echo "Parts of the command couldn't be recognized, including: '-flag'" > /dev/stderr
   exit 1
fi